const parser = require('./parser');
const es = require('./es');
const R = require('ramda');

const createIdFromKeys = keys => Object.keys(keys).map(key => keys[key]).sort().join('#');

const isRecordsIsArray = event => R.propSatisfies(Array.isArray, 'Records', event);

const processRecords = async (handlers, records) => {
  if (records.length === 0) {
    return;
  }
  const record = records[0];
  const action = parser.getAction(record);
  const doc = parser.getDoc(record, action);
  const keys = parser.parseKeys(record);
  const id = createIdFromKeys(keys);

  try {
    await handlers[action](id, doc);
  } catch (e) {
    console.log('Error: ', e);
  }
  processRecords(handlers, records.slice(1));
};

module.exports = config => (event) => {
  if (!isRecordsIsArray(event)) {
    throw new Error('event.Records should be the valid array of records produced by DynamoDB stream.');
  }
  if (event.Records.length === 0) {
    return Promise.resolve([]);
  }

  const { index, type, createIndexIfNeeded = true } = config;
  if (!index) {
    throw new Error('"config.index" should be specified.');
  }
  if (!type) {
    throw new Error('"config.type" should be specified.');
  }

  return es({
    index,
    type,
    createIndexIfNeeded,
  })
    .then(handlers => processRecords(handlers, event.Records));
};
